#!/usr/bin/env python2.7

import os
import sys
import argparse
import pickle

import numpy as np
import matplotlib.pyplot as plt

import parsers
import samples_db
import cnv_struct
import validation
import cnv_db.db as db

def main(args):
    ros = []
        
    p = validation.get_parser_if_valid(args)
    cnvs = p.get_cnvs()

    sample_ids = samples_db.get_sample_ids_for_family(p.family_id)
    algo = None

    for sample in cnvs:
            for chr in cnvs[sample]:
                for cnv in cnvs[sample][chr]:
                    if not algo:
                        algo = cnv.algo
                    dgv_cnv_list = db.query_dgv_overlap(cnv)
                    if len(dgv_cnv_list) == 0:
                        # Didn't find any overlap to DGV.
                        cnv.dgv_ro = 0
                        ros.append(0)
                    else:
                        ro = cnv_struct.ro_to_best_in_list(cnv, dgv_cnv_list)
                        cnv.dgv_ro = ro[0]
                        ros.append(ro[0])

    with open("{}_{}_dgv_overlap.pickle".format(p.family_id, algo), "wb") as f:
        pickle.dump(cnvs, f)
    ros = np.array(ros)

    fig = plt.figure()
    axes = fig.add_subplot(111)
    axes.hist(
                ros,
                color=("#33B5E5", ),
                weights=np.zeros_like(ros) + 1.0 / len(ros)
             )

    axes.set_xlabel("Overlap to DGV")
    axes.set_ylabel("Proportion")

    if args.title != "":
        axes.set_title(args.title)

    plt.show()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description = ("Plots the ROs for the "
                                                    "DGV."))

    parser = validation.add_pickle_args(parser)

    parser.add_argument(
                            "--format",
                            type=str,
                            choices=parsers.__all__,
                            help="The file format for the parser."
                       )

    parser.add_argument(
                            "--base_dir", "-i",
                            type=str,
                            help=("The base directory for the calls. A directory "
                                  "with four subfodlers starting with father, "
                                  "mother, twin1 and twin2 are expected.")
                       )

    parser.add_argument(
                            "--title",
                            type=str,
                            default="",
                            help="Title for the plot."
                       )

    main(parser.parse_args())
