__all__ = ["cnver", "omni25", "pickle", "breakdancer", "rdxplorer", "erds",
           "cnvnator"]

import re
import os

def get_parser_for_algorithm(s):
    """ Gets the appropriate Parser class for a given algorithm. """
    
    # Name of the submodule to get the appropriate Parser class from.
    submod = "parsers.{}".format(s)

    # Dynamically import the module containing the Parser class.
    mod = __import__(submod, fromlist=["Parser"])

    return getattr(mod, "Parser")

def family_to_list(li):
    """Converts a sample => chromo => cnvs dictionary structure to a simple list
       of CNVs for the whole family.

    """

    out = []
    for sample in li:
        for chromo in li[sample]:
            for cnv in li[sample][chromo]:
                out.append(cnv)
    return out

class ParentParser(object):
    def __init__(self, family_root):
        self.family_root = os.path.abspath(family_root)

        # Try to parse a family id from the family_root path
        self.family_id = re.search(r"family_([0-9]+)", family_root)
        if self.family_id:
            self.family_id = self.family_id.group(1)

        ls = os.listdir(family_root)

        # Get sample specific directories.
        try :
            self.twin1_dir = filter(lambda x: x.startswith("twin1"), ls)[0]
            self.twin2_dir = filter(lambda x: x.startswith("twin2"), ls)[0]
            self.father_dir = filter(lambda x: x.startswith("father"), ls)[0]
            self.mother_dir = filter(lambda x: x.startswith("mother"), ls)[0]
        except IndexError:
            raise Exception("Invalid family directory.")

        self.cnvs = {}
        
        self.paths = {
                        "twin1": self.twin1_dir,
                        "twin2": self.twin2_dir,
                        "mother": self.mother_dir,
                        "father": self.father_dir,
                     }

        # Normalize paths
        for sample in self.paths:
            self.paths[sample] = os.path.join(family_root, self.paths[sample])
            self.paths[sample] = os.path.abspath(self.paths[sample])

    def get_cnvs(self):
        raise NotImplementedError()
