# Parser for CNVer output CNV calls.

import re
import os

import cnv_struct
import parsers

class Parser(parsers.ParentParser):
    def __init__(self, family_root):
        super(Parser, self).__init__(family_root)

    def get_cnvs(self):

        # cnvs = { "twin1": {1: [cnvs_for_chr1, ],
        #                    2: [cnvs_for_chr2, ],
        #                           ...
        #                   }
        #           "twin2": ... }

        for sample in self.paths:
            self.cnvs[sample] = {}

            for i in xrange(1, 23):
                self.cnvs[sample][i] = []

                chr_file = os.path.join(
                                            self.family_root,
                                            self.paths[sample],
                                            "calls",
                                            "chr{}.cnvs".format(i)
                                       )

                cols = ["chr", "start", "end", "cnv_type", "doc"]

                with open(chr_file) as f:
                    for line in f:
                        line = line.rstrip("\r\n")
                        fields = dict(zip(cols, line.split("\t")))
                        pos = "{chr}:{start}-{end}".format(**fields)

                        doc = None
                        try:
                            doc = float(fields["doc"])
                        except Exception as e:
                            print ("An error occured while trying to parse "
                                   "the following line: ")
                            print line
                            print
                            print ("From the file:")
                            print f
                            print
                            raise e
                            
                        cnv_builder = {
                                        "pos": pos,
                                        "type": fields["cnv_type"],
                                        "algo": "cnver",
                                        "doc": doc,
                                        "source": sample
                                      }

                        cnv = cnv_struct.cnv(**cnv_builder)
                        self.cnvs[sample][i].append(cnv)

        return self.cnvs

