import re
import os
import collections

import numpy as np

import cnv_struct
import merge_cnvs
import parsers

class Parser(parsers.ParentParser):
    def __init__(self, family_root):
        super(Parser, self).__init__(family_root)

    def get_cnvs(self, confidence_threshold = None):

        for sample in self.paths:
            self.cnvs[sample] = {}
            for i in xrange(1, 23):
                self.cnvs[sample][i] = []

            # Build the path to the calls directory for this sample.
            sample_file = os.path.join(self.family_root, 
                                       self.paths[sample],
                                       "calls")

            # One .events file by calls directory.
            sample_fn = [i for i in os.listdir(sample_file) if
                         i.endswith(".cnv")][0]

            sample_file = os.path.join(sample_file, sample_fn) 

            cols = ["type", "pos", "size", "norm_rd", "p1", "p2", "p3", "p4", 
                    "q0"]

            type_map = {
                            "duplication": "gain",
                            "deletion": "loss",
                       }

            with open(sample_file) as f:
                for line in f:

                    add_to_list = True

                    line = line.rstrip("\r\n")
                    line = line.split("\t")
                    fields = dict(zip(cols, line))
                    
                    confidence = float(fields["p1"])
                    if confidence != 0:
                        confidence = -10.0 * np.log10(confidence)
                    else:
                        # Arbitraire pour eviter la division par zero.
                        confidence = 1000

                    if confidence_threshold:
                        if confidence < confidence_threshold:
                            add_to_list = False
                    
                    chromo = re.search(r"chr([0-9]+):", fields["pos"])
                    if chromo is not None:
                        chromo = chromo.group(1)
                        chromo = int(chromo)

                    else:
                        add_to_list = False

                    if add_to_list:
                        cnv = cnv_struct.cnv(
                                                pos = fields["pos"],
                                                doc = float(fields["norm_rd"]),
                                                type = type_map[fields["type"]],
                                                algo = "cnvnator",
                                                source = sample,
                                                confidence = confidence,
                                            )

                        self.cnvs[sample][chromo].append(cnv)

        # Merge overlapping
        # self.cnvs = merge_cnvs.merge(self.cnvs, threshold = 0)

        return self.cnvs
