import re
import os
import collections

import cnv_struct
import merge_cnvs
import parsers

class Parser(parsers.ParentParser):
    def __init__(self, family_root):
        assert None < -1
        super(Parser, self).__init__(family_root)

    def get_cnvs(self, confidence_threshold = None):

        for sample in self.paths:
            self.cnvs[sample] = {}
            for i in xrange(1, 23):
                self.cnvs[sample][i] = []

            # Build the path to the calls directory for this sample.
            sample_file = os.path.join(self.family_root, 
                                       self.paths[sample],
                                       "calls")

            # One .events file by calls directory.
            sample_fn = [i for i in os.listdir(sample_file) if
                         i.endswith("events")][0]

            sample_file = os.path.join(sample_file, sample_fn) 

            cols = ["chr", "start", "end", "length", "type", "score", 
                    "boundary", "reference_cn", "cn"]

            type_map = {
                            "DUP": "gain",
                            "DEL": "loss",
                       }

            with open(sample_file) as f:
                for line in f:

                    add_to_list = True

                    line = line.rstrip("\r\n")
                    line = line.split("\t")
                    fields = dict(zip(cols, line))
                    
                    start = int(fields["start"])
                    end = int(fields["end"])

                    try:
                        confidence = float(fields["score"])
                    except ValueError:
                        # NAs occur in the file for small deletions.
                        confidence = None

                    cn = int(fields["cn"])

                    chromo = None
                    if fields["chr"] not in ("X", "Y", "MT"):
                        try:
                            chromo = float(fields["chr"])
                        except ValueError:
                            # Couldn't parse a valid chromosome.
                            add_to_list = False
                    else:
                        add_to_list = False

                    if confidence_threshold:
                        if confidence < confidence_threshold:
                            add_to_list = False
                    
                    if add_to_list:
                        cnv = cnv_struct.cnv(
                                                start = start,
                                                end = end,
                                                chr = chromo,
                                                cn = cn,
                                                type = type_map[fields["type"]],
                                                algo = "erds",
                                                source = sample,
                                                confidence = confidence,
                                            )

                        self.cnvs[sample][chromo].append(cnv)

        # Merge overlapping
        # self.cnvs = merge_cnvs.merge(self.cnvs, threshold = 0)

        return self.cnvs
