# Parser for CNVer output CNV calls.

import re
import os

import cnv_struct
import parsers

class Parser(parsers.ParentParser):

    CONFIDENCE_THRESHOLD = 1.96

    def __init__(self, family_root):
        super(Parser, self).__init__(family_root)

    def get_cnvs(self):

        # cnvs = { "twin1": {1: [cnvs_for_chr1, ],
        #                    2: [cnvs_for_chr2, ],
        #                           ...
        #                   }
        #           "twin2": ... }

        for sample in self.paths:
            self.cnvs[sample] = {}

            for i in xrange(1, 23):
                self.cnvs[sample][i] = []

                chr_file = os.path.join(
                                            self.family_root,
                                            self.paths[sample],
                                            "calls",
                                            "chr{}.sum".format(i)
                                       )


                # State: 1 => Deletion; 2 => normal; 3 => duplication.
                # Median is the median normalized read-depth for segment.
                # zstat is the Z-score of the segment compared to the chromosome
                cols = ["segstart", "segend", "state", "length", "cn", "median",
                        "zstat", "chr", "start", "end"]

                with open(chr_file) as f:
                    header = f.readline()
                    for line in f:
                        skip = False
                        line = line.rstrip("\r\n")
                        fields = dict(zip(cols, line.split("\t")))

                        # Convert from scientific notation
                        start = int(float(fields["start"]))
                        end = int(float(fields["end"]))

                        pos = "chr{chr}:{start}-{end}".format(
                            chr = fields["chr"],
                            start = start,
                            end = end
                        )

                        doc = float(fields["median"])
                        cn = int(fields["cn"])
                        confidence = abs(float(fields["zstat"]))

                        if fields["chr"] not in [str(e) for e in xrange(1, 23)]:
                            skip = True

                        if confidence < Parser.CONFIDENCE_THRESHOLD:
                            skip = True

                        cnv_type = ""
                        if cn < 2:
                            cnv_type = "loss"
                        elif cn > 2:
                            cnv_type = "gain"
                        else:
                            print cn

                        if not skip:
                            cnv_builder = {
                                            "pos": pos,
                                            "type": cnv_type,
                                            "algo": "rdxplorer",
                                            "doc": doc,
                                            "cn": cn,
                                            "source": sample,
                                            "confidence": confidence,
                                          }

                            cnv = cnv_struct.cnv(**cnv_builder)
                            self.cnvs[sample][i].append(cnv)

        return self.cnvs
