#!/usr/bin/env python2.7

# Computes the average CNV size and std.

import sys
import argparse
import numpy as np
import matplotlib.pyplot as plt

import validation
import parsers
import cnv_struct

def main(args):
    # List of 2D vectors of size, doc.
    size_doc_gain = []
    size_doc_loss = []
    locs = []
    source = []

    p = validation.get_parser_if_valid(args)

    # Get the CNVs
    cnvs = p.get_cnvs()

    for sample in cnvs:
        other = None
        if sample == "twin1":
            other = "twin2"
        elif sample == "twin2":
            other = "twin1"

        if other:
            for chr in cnvs[sample]:
                for cnv in cnvs[sample][chr]:
                    min_ro = min(cnv_struct.ro_to_best_in_list(
                                                                cnv,
                                                                cnvs[other][chr]
                                                              ))

                    father_ro = cnv_struct.ro_to_best_in_list(
                                                                cnv,
                                                                cnvs["father"][chr]
                                                             )

                    mother_ro = cnv_struct.ro_to_best_in_list(
                                                                cnv,
                                                                cnvs["mother"][chr]
                                                             )

                    max_min_parent_ro = max([min(father_ro), min(mother_ro)]) 
                    
                    store = True
                    if args.ro_over and args.enforce_parent_ro:
                        store = (min_ro >= args.ro_over and
                                 max_min_parent_ro >= args.ro_over)

                    elif args.ro_over:
                        store = min_ro >= args.ro_over
                
                    elif args.enforce_parent_ro and not args.ro_over:
                        sys.exit(("When the `--enforce_parent_ro` option is "
                                  "used, it is necessary to provide a "
                                  "`--ro_over` threshold for this program "
                                  "to filter out CNVs accordingly."))

                    if store:
                        if cnv.type == "gain":
                            size_doc_gain.append(
                                np.array([cnv.size / 1000.0, cnv.doc])
                            )
                        else:
                            size_doc_loss.append(
                                np.array([cnv.size / 1000.0, cnv.doc])
                            )

                        locs.append(cnv.get_pos())
                        source.append(cnv.source)
                    
    size_doc_gain = np.array(size_doc_gain)
    size_doc_loss = np.array(size_doc_loss)

    if args.plotsame:
        plot_in_same_plot(size_doc_gain, size_doc_loss)

    else:
        plot_in_subplots(size_doc_gain, size_doc_loss)

def plot_in_same_plot(gain, loss):
    f = plt.figure()
    plt.plot(gain[:, 0], gain[:, 1], 'r.')
    plt.plot(loss[:, 0], loss[:, 1], 'b.')
    plt.xlabel("Taille (kb)")
    plt.ylabel("DOC")
    plt.axhline(y = 1, color = "#000000")
    plt.xlim(0, 300)
    plt.show()


def plot_in_subplots(gain, loss):
    f, (ax1, ax2) = plt.subplots(1, 2, sharey=True)

    ax1.plot(
                gain[:, 0],
                gain[:, 1],
                'k.'
           )

    ax2.plot(
                loss[:, 0],
                loss[:, 1],
                'k.'
           )

    ax1.set_xlabel("Taille (kb)")
    ax1.set_ylabel("DOC")
    ax1.set_title("gain")

    ax1.axhline(y = 1)

    ax2.set_xlabel("Taille (kb)")
    ax2.set_ylabel("DOC")
    ax2.set_title("loss")

    ax2.axhline(y = 1)
    
    if args.save is None:
        plt.show()
    else:
        plt.savefig(args.save)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Computes the average and std CNV size.")

    parser = validation.add_pickle_args(parser)

    parser.add_argument("--format",
                        type=str,
                        choices=parsers.__all__,
                        help="The file format for the parser.")

    parser.add_argument("--base_dir",
                        type=str,
                        help=("The base directory for the calls. A directory with "
                              "four subfolders starting with father, mother, twin1 "
                              "and twin2 are expected."))

    parser.add_argument("--ro_over",
                        default=None,
                        type=float,
                        help="Overlap threshold for displayed CNVs."
                       )

    parser.add_argument("--enforce_parent_ro",
                        action="store_true", 
                        help=("Applies the ro_over threshold to the other twin "
                              "AND the parents.")
                       )

    parser.add_argument("--plotsame",
                        action="store_true",
                        help="Plot in one or two windows."
                       )

    parser.add_argument("--save",
                    type=str,
                    help="Filename of the output graph file.",
                    default=None
                   )

    main(parser.parse_args()) 
