#!/usr/bin/env python2.7

import os
import pickle

import parsers
import merge_cnvs
import venn
import mendelian

THRESHOLD = 0.7

def main():

    families = ["family_1121", "family_1207", "family_1286", "family_1301", 
                "family_1323", "family_1389", "family_1443", "family_1480", 
                "family_1490", "family_1652"]

    samples = ["father", "mother", "twin1", "twin2"]

    base = os.path.abspath(os.path.join(os.path.dirname(__file__), '../data'))
    algos = ["cnver", "break-dancer", "cnvnator", "erds"]

    counts_for_all_samples = {}

    for family in families:
        # sample -> algo -> list of cnvs
        cnvs_per_sample = {
            'father': {},
            'mother': {},
            'twin1': {},
            'twin2': {}                
        }

        big_li = {}
        for algo in algos:

            base_dir = os.path.join(base, algo, family)

            # Parse the CNV
            if algo == 'break-dancer': algo = 'breakdancer'
            p = parsers.get_parser_for_algorithm(algo)
            p = p(base_dir)
            cnvs = p.get_cnvs()
            
            # Merge
            cnvs = merge_cnvs.merge(cnvs)

            # Filter inherited
            cnvs = mendelian.get_inherited(cnvs, 0.7, 0.7)[0]

            big_li[algo] = cnvs

        li = [big_li['cnver'], big_li['breakdancer'], big_li['cnvnator'], big_li['erds']]
        print 'Generating inherited Venn for {}'.format(family)
        counts = venn.position_based_venn(li, THRESHOLD)
        print

        venn.create_venn_from_dict(
            counts, 
            "venns/Inherited_venn_{}.svg".format(family)
        )

        # Add the counts to the global counts.
        for k in counts:
            if counts_for_all_samples.get(k) is None:
                counts_for_all_samples[k] = counts[k]
            else:
                counts_for_all_samples[k] += counts[k]

    # Write the global Venn
    venn.create_venn_from_dict(counts_for_all_samples, 'venns/Inherited_venn_all_samples.svg')

if __name__ == "__main__":
    main()
