#!/usr/bin/env python2.7

import sys
import os
import os.path
import sqlite3

DB_FILE = os.path.join(os.path.split(os.path.abspath(__file__))[0], "samples.db")

def create_db(trios_file, coverage_file, delete=False):
    db_file = DB_FILE
    if delete and os.path.isfile(db_file):
        os.remove(db_file)
        
    elif os.path.isfile(db_file):
        raise Exception("Database file already exists. run with "
                        "delete=True to overwrie or delete the samples.db "
                        "file manually.")

    # Create the database
    conn = sqlite3.connect(db_file)
    c = conn.cursor()
    c.execute(("CREATE TABLE quatuors (id text, family text, "
               "status text, avg_coverage float)"))

    # Parse the coverage file into a dict.
    coverage = {}
    with open(coverage_file) as f:
        for line in f:
            line = line.rstrip("\r\n")
            line = line.split(",")
            # Sample -> coverage dict.
            coverage[line[0]] = line[1]

    # Insert the corresponding rows
    samples_info = []
    with open(trios_file) as f:
        header = f.readline()
        header = header.rstrip("\r\n")
        header = header.split("\t")
        for line in f:
            line = line.rstrip("\r\n")
            line = line.split("\t")
            sample_id = line[header.index("#ID")]
            family_id = line[header.index("Family")]
            status    = line[header.index("Link")]
            cov = coverage[sample_id]
            status = status.lower()
            if line[header.index("Project")] == "Twins":
                samples_info.append((sample_id, family_id, status, cov))

    c.executemany("INSERT INTO quatuors VALUES (?,?,?,?)", samples_info)

    # Save and exit
    conn.commit()
    conn.close()

def get_sample_ids_for_family(f):
    family = {}
    c = __get_cursor()
    response = c.execute("SELECT id, status FROM quatuors WHERE family=?", (f, ))
    for sample_id, status in response:
        family[status] = sample_id
    return family

def get_coverage_for_sample(s):
    c = __get_cursor()
    coverage = c.execute("SELECT avg_coverage FROM quatuors WHERE id=?", (s, )).fetchone()
    return coverage[0]


def __get_cursor():
    conn = sqlite3.connect(DB_FILE)
    c = conn.cursor()
    return c
    
if __name__ == "__main__":
    if len(sys.argv) != 3:
        print "Usage: {} trios coverage".format(__file__)
        print ("Where `trios` is the `complete_trios` file to create the database "
               "and `coverage` is the coverage csv file to get the average coverage "
               "values from.")
        print "For examples, see the `sample_db_files` directory."
        sys.exit()

    create_db(sys.argv[1], sys.argv[2])
