#!/usr/bin/env python2.7

import sys
import os.path

import parsers
import parsers.cnvpickle

def get_parser_if_valid(args):
    if args.base_dir:
        if not os.path.isdir(args.base_dir):
            sys.exit("The --base_dir should be the calls directory.")
        elif not args.format:
            sys.exit("You need to provide a --format.")

        p = parsers.get_parser_for_algorithm(args.format)
        # Initialize the Parser.
        p = p(args.base_dir)

    elif args.pickle:
        if not args.family:
            sys.exit("You need to manually provide the family id with --family.")
        p = parsers.cnvpickle.Parser
        p = p(args.pickle, args.family)

    else:
        sys.exit("Either provide a --pickle file or a --base_dir.")

    return p

def add_pickle_args(parser):
    parser.add_argument(
                            "--pickle",
                            type=str,
                            help=("Load CNVs from serialized pickle data.")
                       )

    parser.add_argument(
                            "--family",
                            type=str,
                            help="Family ID for pickled data."
                       )

    return parser

def add_dir_args(parser):
    parser.add_argument("--format",
                        type=str,
                        choices=parsers.__all__,
                        help="The file format for the parser.")

    parser.add_argument("--base_dir",
                        type=str,
                        help=("The base directory for the calls. A directory with "
                              "four subfolders starting with father, mother, twin1 "
                              "and twin2 are expected."))
    return parser
